<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\static_node\Service\StaticNodeHooks;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\RedirectCommand;

/**
 * Provides a confirmation form for bulk deleting static files.
 */
class StaticFilesBulkDeleteConfirmForm extends FormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;
  
  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The selected file IDs.
   *
   * @var array
   */
  protected $fileIds = [];

  /**
   * Constructs a StaticFilesBulkDeleteConfirmForm object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    StaticNodeHooks $static_node_service,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->staticNodeService = $static_node_service;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('static_node.hooks'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_files_bulk_delete_confirm';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Ensure we have the proper libraries
    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
    $form['#attached']['library'][] = 'core/drupal.ajax';
    
    // Set proper form ID and wrapper
    $form['#id'] = 'static-files-bulk-delete-form';
    $form['#prefix'] = '<div id="static-files-bulk-delete-form">';
    $form['#suffix'] = '</div>';
    
    // Get selected items from tempstore
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_delete');
    $selected_items = $tempstore->get($this->currentUser->id());
    
    if (empty($selected_items)) {
      $this->messenger->addError($this->t('No files were selected for deletion.'));
      return $form;
    }
    
    // Store the file IDs
    $this->fileIds = $selected_items;
    $form_state->set('selected_items', $selected_items);
    
    // Load the file entities
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($selected_items);
    
    if (empty($files)) {
      $this->messenger->addError($this->t('No files were found for deletion.'));
      return $form;
    }
    
    // Add a dedicated container for displaying messages in the modal
    $form['messages_container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'static-files-messages',
        'class' => ['messages-container'],
      ],
    ];
    
    $items = [];
    foreach ($files as $file) {
      // Try to find the source node if available
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $items[] = [
              '#markup' => $node->label(),
            ];
          }
        }
        catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error loading source node: @error', ['@error' => $e->getMessage()]);
        }
      }
    }
    
    $form['title'] = [
      '#type' => 'markup',
      '#markup' => '<h3>' . $this->t('Delete Static Files') . '</h3>',
    ];
    
    $form['description'] = [
      '#markup' => '<p>' . $this->t('This will delete the static HTML versions of these pages. The original content will not be affected, but anonymous users will see the dynamic version of the pages instead.') . '</p>',
    ];
    
    $form['files'] = [
      '#theme' => 'item_list',
      '#items' => $items,
      '#title' => $this->formatPlural(count($files), '1 file selected for deletion:', '@count files selected for deletion:'),
    ];
    
    $form['nodes_items'] = [
      '#type' => 'hidden',
      '#default_value' => implode(',', $this->fileIds),
    ];
    
    // Add actions with proper AJAX configuration
    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete Static Files'),
      '#button_type' => 'primary',
      '#ajax' => [
        'callback' => '::ajaxSubmit',
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Deleting static files...'),
        ],
      ],
    ];
    
    $form['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#ajax' => [
        'callback' => '::closeModal',
      ],
      '#attributes' => ['class' => ['dialog-cancel']],
    ];
    
    return $form;
  }

  /**
   * AJAX callback to close the modal.
   */
  public function closeModal(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

  /**
   * AJAX submit callback for the form.
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state) {
    // Special handling for AJAX submission
    $form_state->setRebuild(FALSE);
    $this->submitForm($form, $form_state);
    
    $response = new AjaxResponse();
    
    // Add command to close the modal dialog
    $response->addCommand(new CloseModalDialogCommand());
    
    // Get the destination URL for redirection
    $tempstore = $this->tempStoreFactory->get('static_node');
    $destination = $tempstore->get('bulk_delete_destination');
    
    // Add the redirect command
    if ($destination) {
      $tempstore->delete('bulk_delete_destination');
      $response->addCommand(new RedirectCommand($destination));
    }
    else {
      // Fallback to files management page if no destination stored
      $url = Url::fromRoute('static_node.files_management')->toString();
      $response->addCommand(new RedirectCommand($url));
    }
    
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->loggerFactory->get('static_node')->debug('Delete form submission started');
    
    // Get selected items
    $selected_items = $form_state->get('selected_items');
    
    if (empty($selected_items)) {
      $nodes_items = $form_state->getValue('nodes_items');
      $this->loggerFactory->get('static_node')->debug('Trying to get items from nodes_items field: @items', ['@items' => $nodes_items]);
      
      if (!empty($nodes_items)) {
        $selected_items = explode(',', $nodes_items);
      }
    }
    
    if (empty($selected_items)) {
      $this->loggerFactory->get('static_node')->warning('No items selected for static file deletion');
      $this->messenger->addError($this->t('No items were selected for static file deletion.'));
      return;
    }
    
    $this->loggerFactory->get('static_node')->debug('Processing @count items for deletion', ['@count' => count($selected_items)]);
    
    // Process each file
    $processed = 0;
    $errors = 0;
    
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($selected_items);
    
    foreach ($files as $file) {
      try {
        // Try to find the source node if available
        if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
          $nid = $file->field_static_node->target_id;
          
          try {
            $node = $this->entityTypeManager->getStorage('node')->load($nid);
            if ($node) {
              $this->loggerFactory->get('static_node')->debug('Deleting static version for node @nid (@title)', [
                '@nid' => $node->id(),
                '@title' => $node->getTitle()
              ]);
              
              // Create a temporary form array and form state for the static node service
              $temp_form = [];
              $temp_form_state = new \Drupal\Core\Form\FormState();
              $temp_form_state->set('node', $node);
              
              // Delete the static version using the service
              $this->staticNodeService->deleteStaticVersion($temp_form, $temp_form_state);
              $processed++;
              $this->loggerFactory->get('static_node')->info('Static file deleted successfully for node @nid', ['@nid' => $nid]);
            }
            else {
              $this->loggerFactory->get('static_node')->warning('Node @nid not found, attempting to delete the file directly', ['@nid' => $nid]);
              // If node not found, try to delete the file directly
              $file->delete();
              $processed++;
            }
          }
          catch (\Exception $e) {
            $errors++;
            $this->loggerFactory->get('static_node')->error('Error deleting static file for node @nid: @error', [
              '@nid' => $nid,
              '@error' => $e->getMessage()
            ]);
          }
        }
        else {
          // If no node reference, delete the file directly
          $this->loggerFactory->get('static_node')->info('File @fid has no source node, deleting directly', ['@fid' => $file->id()]);
          $file->delete();
          $processed++;
        }
      }
      catch (\Exception $e) {
        $errors++;
        $this->loggerFactory->get('static_node')->error('Error deleting file @fid: @error', [
          '@fid' => $file->id(),
          '@error' => $e->getMessage()
        ]);
      }
    }
    
    $this->loggerFactory->get('static_node')->info('Deletion completed. @processed deleted, @errors failed', [
      '@processed' => $processed,
      '@errors' => $errors
    ]);
    
    // Store messages in the session so they persist
    if ($processed > 0) {
      $message = $this->formatPlural($processed, 
        '1 static file was successfully deleted.', 
        '@count static files were successfully deleted.'
      );
      $this->messenger->addStatus($message);
    }
    
    if ($errors > 0) {
      $message = $this->formatPlural($errors,
        'Failed to delete 1 static file. Check the logs for more information.',
        'Failed to delete @count static files. Check the logs for more information.'
      );
      $this->messenger->addError($message);
    }
    
    if ($processed == 0 && $errors == 0) {
      $message = $this->t('No static files were deleted.');
      $this->messenger->addWarning($message);
    }
    
    // Clear the tempstore data
    $this->tempStoreFactory->get('static_node_bulk_delete')->delete($this->currentUser->id());
  }
}