<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\static_node\Service\StaticNodeHooks;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Form\FormState;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Form\FormBuilderInterface;

/**
 * Provides a confirmation form for bulk generating static files.
 */
class StaticFilesBulkGenerateConfirmForm extends FormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;
  
  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The selected file IDs.
   *
   * @var array
   */
  protected $fileIds = [];

  /**
   * Constructs a StaticFilesBulkGenerateConfirmForm object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    StaticNodeHooks $static_node_service,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->staticNodeService = $static_node_service;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('static_node.hooks'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_files_bulk_generate_confirm';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $selected_items = NULL) {
    
    // Check if the user has permission to generate static files
    if (!$this->currentUser->hasPermission('generate static node')) {
      $this->logger('static_node')->error('You do not have permission to generate static files.');
      $this->messenger->addError($this->t('You do not have permission to generate static files.'));
      return $form;
    }
    $form['#attached']['library'][] = 'core/drupal.dialog';
    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    // Set proper form ID and wrapper
    $form['#prefix'] = '<div id="static-files-bulk-generate-wrapper">';
    $form['#suffix'] = '</div>';
    
    // Get selected items from tempstore
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_generate');
    $selected_items = $tempstore->get($this->currentUser->id());
    
    if (empty($selected_items)) {
      $this->logger('static_node')->error('No files were selected for static file generation.');
      $this->messenger->addError($this->t('No files were selected for static file generation.'));
      return $form;
    }
    
    // Store the file IDs
    $this->fileIds = $selected_items;
    $form_state->set('selected_items', $selected_items);
    
    // Load the file entities
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($selected_items);
    
    if (empty($files)) {
      $this->logger('static_node')->error('No files were found for static file generation.');
      $this->messenger->addError($this->t('No files were found for static file generation.'));
      return $form;
    }
    
    // Add a dedicated container for displaying messages in the modal
    $form['messages_container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'static-files-messages',
        'class' => ['messages-container'],
      ],
    ];
    
    $items = [];
    foreach ($files as $file) {
      // Cehck if file is file entity
      if (!$file instanceof \Drupal\file\FileInterface) {
        continue;
      }
      // Try to find the source node if available
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $items[] = [
              '#markup' => $node->label(),
            ];
          }
        }
        catch (\Exception $e) {
          $this->logger('static_node')->error('Error loading node @nid: @error', [
            '@nid' => $nid,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
    
    $form['title'] = [
      '#type' => 'markup',
      '#markup' => '<h3>' . $this->t('Generate Static Files') . '</h3>',
    ];
    
    $form['description'] = [
      '#markup' => '<p>' . $this->t('This will create static HTML versions of these pages which will be served to anonymous users for better performance.') . '</p>',
    ];
    
    $form['files'] = [
      '#theme' => 'item_list',
      '#items' => $items,
      '#title' => $this->formatPlural(count($files), '1 file selected for static generation:', '@count files selected for static generation:'),
    ];
    
    $form['nodes_items'] = [
      '#type' => 'hidden',
      '#default_value' => $selected_items,
    ];
    
    // Add actions with proper AJAX configuration
    $form['actions'] = [
      '#type' => 'actions',
      '#prefix' => '<div class="form-actions">',
      '#suffix' => '</div>',
    ];

    $form['actions']['submit'] = [
      '#type' => 'button',
      '#value' => $this->t('Generate Static Files'),
      '#button_type' => 'primary',
      '#attributes' => ['class' => ['button--primary', 'use-ajax']],
      '#limit_validation_errors' => [],
      '#ajax' => [
        'url' => Url::fromRoute('static_node.bulk_files_generate_confirm'),
        'callback' => [$this, 'generateStaticFiles'],
        'event' => 'click',
        'effect' => 'fade',
        'options' => [
          'query' => [
            FormBuilderInterface::AJAX_FORM_REQUEST => TRUE,
          ],
        ],
      ],
    ];
    
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#value' => $this->t('Cancel'),
      '#attributes' => [
        'class' => ['dialog-cancel'],
      ],
    ];
    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: buildForm() called.');
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // No validation needed for this form
    // but we can check if the user has permission to generate static files

    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: validateForm() called.');
  }

  /**
   * AJAX callback to close the modal.
   */
  public function closeModal(array &$form, FormStateInterface $form_state) {
    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: closeModal() called.');
    $response = new AjaxResponse();
    $response->addCommand(new CloseDialogCommand('#static-files-bulk-form-modal'));
    $this->logger('static_node')->debug('closeModal: buildForm() called.');
    return $response;
  }

  /**
   * AJAX submit callback for the form.
   */
  public function generateStaticFiles(array &$form, FormStateInterface $form_state) {
    // Get the file IDs from the form state
    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: generateStaticFiles() called.');
    $file_ids = $form_state->getValue('nodes_items');
    $response = new AjaxResponse();
    $message = '';

    if (is_string($file_ids)) {
      $file_ids = explode(' ', $file_ids);
    }
    elseif (!is_array($file_ids)) {
      $file_ids = [];
    }
    
    if (empty($file_ids)) {
      $message .= $this->t('No files were selected for static file generation.');
    }
    
    $success_count = 0;
    $fail_count = 0;
    
    foreach ($file_ids as $file_id) {
      if (!is_numeric($file_id) || empty($file_id)) {
        continue;
      }
      
      // Try to find the source node if available
      $file = $this->entityTypeManager->getStorage('file')->load($file_id);
      if ($file && $file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $temp_form = [];
            $node_form_state = new FormState();
            $node_form_state->set('node', $node);
            $result = $this->staticNodeService->generateStaticVersion($temp_form, $node_form_state);
            if ($result) {
              $success_count++;
            }
            else {
              $fail_count++;
            }
          }
        }
        catch (\Exception $e) {
          $fail_count++;
        }
      }
    }
    
    // Provide feedback
    if ($success_count > 0) {
      $message .= $this->formatPlural($success_count, 
        '1 static file was generated successfully. ', 
        '@count static files were generated successfully. '
      );
    }
    if ($fail_count > 0) {
      $message .= $this->formatPlural($fail_count, 
        '1 file failed to generate. ', 
        '@count files failed to generate. '
      );
    }
    
    // Clean up the tempstore
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_generate');
    $tempstore->delete($this->currentUser->id());


    $response->addCommand(new CloseDialogCommand('#static-files-bulk-form-modal'));

    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: generateStaticFiles() called.');
    
    $this->messenger->addStatus($message, 'status', TRUE);

    // Redirect to files page
    $url = Url::fromRoute('static_node.files_management');
    $response->addCommand(new RedirectCommand($url->toString()));

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    //$form_state->setRebuild(TRUE);
    $this->logger('static_node')->debug('StaticFilesBulkGenerateConfirmForm: submitForm() called.');
  }
}