<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Url;

/**
 * Form handler for static files bulk operations.
 */
class StaticFilesBulkOperationsForm extends FormBase {

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a new StaticFilesBulkOperationsForm.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(
    MessengerInterface $messenger,
    AccountProxyInterface $current_user,
    PrivateTempStoreFactory $temp_store_factory,
    FormBuilderInterface $form_builder
  ) {
    $this->messenger = $messenger;
    $this->currentUser = $current_user;
    $this->tempStoreFactory = $temp_store_factory;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('form_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_bulk_operations_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['bulk_operations']['#id'] = 'static-files-bulk-form';
    $form['bulk_operations']['#attached']['library'][] = 'core/drupal.dialog';
    $form['bulk_operations']['#attached']['library'][] = 'core/drupal.dialog.ajax';
    $form['bulk_operations'] = [
      '#type' => 'details',
      '#title' => $this->t('Bulk operations'),
      '#open' => TRUE,
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['bulk_operations']['status'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['bulk-operations__item', 'bulk-operations__item--status'],
      ],
      '#markup' => $this->t('No items selected'),
    ];
    $form['bulk_operations']['action'] = [
      '#type' => 'select',
      '#title' => $this->t('With selection'),
      '#title_display' => 'invisible',
      '#options' => [
        'delete' => $this->t('Delete'),
        'regenerate' => $this->t('Regenerate'),
      ],
      '#empty_option' => $this->t('- Select action -'),
      '#default_value' => $this->t('- Select action -'),
    ];

    // Add submit button with proper AJAX configuration
    $form['bulk_operations']['submit'] = [
      '#type' => 'button',
      '#value' => $this->t('Apply'),
      '#ajax' => [
        'callback' => [$this, 'submitAjaxCallback'],
        'wrapper' => 'static-files-bulk-form',
        'effect' => 'fade',
      ],
    ];
    
    // Hidden field to store selected items
    $form['bulk_operations']['selected_items'] = [
      '#type' => 'hidden',
      '#default_value' => $form_state->getValue('selected_items'),
      '#attributes' => ['id' => 'selected-items'],
    ];
    
    return $form['bulk_operations'];
  }

  /**
   * AJAX callback for the submit button.
   */
  public function submitAjaxCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $selected_items = $form_state->getValue('selected_items');
    $action = $form_state->getValue('action');
    
    if (empty($action)) {
      // No action selected, show an error message
      $response->addCommand(new MessageCommand(
        $this->t('Please select an action to perform.'),
        '#bulk-operations-message',
        ['type' => 'error']
      ));
      return $response;
    }
    
    if (empty($selected_items)) {
      // No items selected, show an error message
      $response->addCommand(new MessageCommand(
        $this->t('No items selected. Please select at least one item.'),
        '#bulk-operations-message',
        ['type' => 'error']
      ));
      return $response;
    }
    // Convert selected items to an array
    $selected_items = explode(',', $selected_items);
    // Determine which form to use
    $confirm_form = NULL;
    $title = '';
    
    switch ($action) {
      case 'delete':
        $this->tempStoreFactory->get('static_node_bulk_delete')->set($this->currentUser->id(), $selected_items);
        // Store the current destination to return to after processing
        $request = $this->getRequest();
        $destination = $request->getPathInfo();
        $query = $request->getQueryString();
        if ($query) {
          $destination .= '?' . $query;
        }
        $this->tempStoreFactory->get('static_node')->set('bulk_destination', $destination);
        
        $confirm_form = $this->formBuilder->getForm('Drupal\static_node\Form\StaticFilesBulkDeleteConfirmForm', $selected_items);
        $title = $this->t('Confirm deletion');
        break;
        
      case 'regenerate':
        $this->tempStoreFactory->get('static_node_bulk_generate')->set($this->currentUser->id(), $selected_items);
        // Store the current destination to return to after processing
        $request = $this->getRequest();
        $destination = $request->getPathInfo();
        $query = $request->getQueryString();
        if ($query) {
          $destination .= '?' . $query;
        }
        $this->tempStoreFactory->get('static_node')->set('bulk_generate_destination', $destination);
        
        $confirm_form['form'] = $this->formBuilder->getForm('Drupal\static_node\Form\StaticFilesBulkGenerateConfirmForm', $selected_items);
        $title = $this->t('Confirm regeneration');
        break;
    }
    
    

    if ($confirm_form) {
      $options = [
        'width' => 'auto',
        'dialogClass' => 'static-files-bulk-form-modal',
        'closeOnEscape' => TRUE,
        'modal' => TRUE,
      ];
      // Create the AJAX command to open the modal dialog with the rendered form
      $response->addCommand(new OpenDialogCommand(
        '#static-files-bulk-form-modal',
        $this->t('Are you sure you want to perform this action?'),
        $confirm_form,
        $options
      ));
    }
    
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // This is a fallback for non-JS situations
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $selected_items = $form_state->getValue('selected_items');
    $action = $form_state->getValue('action');
    
    if (empty($action)) {
      $form_state->setErrorByName('action', $this->t('Please select an action to perform.'));
    }
    
    if (empty($selected_items)) {
      $form_state->setErrorByName('selected_items', $this->t('No items selected. Please select at least one item.'));
    }
  }
}